package cn.com.qmth.examcloud.core.examwork.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.commons.api.request.SyncCourseReq;
import cn.com.qmth.examcloud.commons.api.request.SyncExamReq;
import cn.com.qmth.examcloud.commons.api.request.SyncExamStudentReq;
import cn.com.qmth.examcloud.commons.api.request.SyncOrgReq;
import cn.com.qmth.examcloud.commons.api.request.SyncSpecialtyReq;
import cn.com.qmth.examcloud.commons.api.request.SyncStudentReq;
import cn.com.qmth.examcloud.commons.api.request.SyncUserReq;
import cn.com.qmth.examcloud.commons.api.response.SyncCourseResp;
import cn.com.qmth.examcloud.commons.api.response.SyncExamResp;
import cn.com.qmth.examcloud.commons.api.response.SyncExamStudentResp;
import cn.com.qmth.examcloud.commons.api.response.SyncOrgResp;
import cn.com.qmth.examcloud.commons.api.response.SyncSpecialtyResp;
import cn.com.qmth.examcloud.commons.api.response.SyncStudentResp;
import cn.com.qmth.examcloud.commons.api.response.SyncUserResp;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.examwork.dao.ExamCourseRelationRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月18日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.examwork}" + "dataSync")
public class HandleSyncCloudServiceProvider extends ControllerSupport
		implements
			HandleSyncCloudService {

	private static final long serialVersionUID = 9122543745597481498L;

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamCourseRelationRepo examCourseRelationRepo;

	@ApiOperation(value = "同步课程")
	@PostMapping("syncCourse")
	@Override
	public SyncCourseResp syncCourse(@RequestBody SyncCourseReq req) {
		String courseName = req.getName();
		Long courseId = req.getId();
		String courseLevel = req.getLevel();
		Long rootOrgId = req.getRootOrgId();
		Boolean enable = req.getEnable();

		if (StringUtils.isBlank(courseName)) {
			throw new StatusException("B-100001", "courseName is null");
		}
		if (null == courseId) {
			throw new StatusException("B-100002", "courseId is null");
		}
		if (StringUtils.isBlank(courseLevel)) {
			throw new StatusException("B-100003", "courseLevel is null");
		}
		if (null == rootOrgId) {
			throw new StatusException("B-100004", "rootOrgId is null");
		}

		examStudentRepo.updateCourse(courseName, courseLevel, courseId);

		examCourseRelationRepo.updateCourse(courseName, courseLevel, enable, courseId);

		SyncCourseResp resp = new SyncCourseResp();
		return resp;
	}

	@ApiOperation(value = "同步机构")
	@PostMapping("syncOrg")
	@Override
	public SyncOrgResp syncOrg(@RequestBody SyncOrgReq req) {
		return null;
	}

	@ApiOperation(value = "同步学生")
	@PostMapping("syncStudent")
	@Override
	public SyncStudentResp syncStudent(@RequestBody SyncStudentReq req) {
		String name = req.getName();
		Long id = req.getId();
		String studentCode = req.getStudentCode();

		if (StringUtils.isBlank(name)) {
			throw new StatusException("B-100001", "name is null");
		}
		if (null == id) {
			throw new StatusException("B-100002", "id is null");
		}

		examStudentRepo.updateStudent(name, studentCode, id);
		return null;
	}

	@ApiOperation(value = "同步考生")
	@PostMapping("syncExamStudent")
	@Override
	public SyncExamStudentResp syncExamStudent(@RequestBody SyncExamStudentReq req) {
		return null;
	}

	@ApiOperation(value = "同步专业")
	@PostMapping("syncSpecialty")
	@Override
	public SyncSpecialtyResp syncSpecialty(@RequestBody SyncSpecialtyReq req) {
		return null;
	}

	@ApiOperation(value = "同步考试")
	@PostMapping("syncExam")
	@Override
	public SyncExamResp syncExam(@RequestBody SyncExamReq req) {
		return null;
	}

	@ApiOperation(value = "同步考试")
	@PostMapping("syncUser")
	@Override
	public SyncUserResp syncUser(@RequestBody SyncUserReq req) {
		return null;
	}

}
