package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.commons.base.helpers.poi.ExcelReader;
import cn.com.qmth.examcloud.commons.base.util.DateUtil;
import cn.com.qmth.examcloud.commons.base.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.commons.web.enums.DataExecutionStatus;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgSettingsRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgSettingsEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamOrgSettingsInfo;
import cn.com.qmth.examcloud.task.api.DataSyncCloudService;
import cn.com.qmth.examcloud.task.api.request.SyncExamReq;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class ExamServiceImpl implements ExamService {
	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	ExamOrgSettingsRepo examOrgSettingsRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@Autowired
	ExamOrgPropertyRepo examOrgPropertyRepo;

	@Autowired
	DataSyncCloudService dataSyncCloudService;

	private static final String[] EXAM_ORG_SETTINGS_EXCEL_HEADER = new String[]{"学习中心ID", "学习中心代码",
			"学习中心名称", "是否可以考试", "开始考试时间", "结束考试时间"};

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examName
	 * @param rootOrgId
	 * @return
	 */
	public ExamEntity findExamByNameAndRootOrgId(String examName, Long rootOrgId) {
		ExamEntity exam = new ExamEntity();
		exam.setName(examName);
		exam.setRootOrgId(rootOrgId);
		return examRepo.findOne(Example.of(exam));
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExam(cn.com.
	 * qmth.examcloud.core.examwork.service.bean.ExamInfo)
	 */
	@Override
	public ExamEntity saveExam(ExamInfo examInfo, DataExecutionStatus es) {

		if (null == examInfo.getExamType()) {
			throw new StatusException("E-001005", "考试类型为空");
		}

		if (StringUtils.isBlank(examInfo.getName())) {
			throw new StatusException("E-001005", "考试名称为空");
		}

		if (null == examInfo.getRootOrgId()) {
			throw new StatusException("E-001005", "rootOrgId is null");
		}

		GetOrgReq getOrgReq = new GetOrgReq();
		getOrgReq.setOrgId(examInfo.getRootOrgId());
		GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
		OrgBean rootOrg = getOrgResp.getOrg();

		Map<String, String> properties = examInfo.getProperties();
		Map<DynamicEnum, String> map = checkAndGetExamProperties(properties);

		ExamEntity exam = null;

		DataExecutionStatus realStatus = null;
		// 更新
		if (es.equals(DataExecutionStatus.UPDATE)) {
			if (null != examInfo.getId()) {
				exam = examRepo.findOne(examInfo.getId());
				if (null == exam) {
					throw new StatusException("E-002001", "id is wrong");
				}
			} else {
				throw new StatusException("E-002002", "id is null");
			}
			if (!exam.getRootOrgId().equals(examInfo.getRootOrgId())) {
				throw new StatusException("E-002003", "rootOrgId is wrong");
			}
			if (!exam.getExamType().equals(examInfo.getExamType())) {
				throw new StatusException("E-002100", "examType is wrong");
			}

			ExamEntity tempExam = examRepo.findByNameAndRootOrgId(examInfo.getName(),
					examInfo.getRootOrgId());
			if (null != tempExam && !tempExam.getId().equals(examInfo.getId())) {
				throw new StatusException("E-002005", "考试名称已存在");
			}

			realStatus = DataExecutionStatus.UPDATE;
		}
		// 创建
		else if (es.equals(DataExecutionStatus.CREATION)) {
			if (null != examInfo.getId()) {
				throw new StatusException("E-002004", "id is needless");
			}
			ExamEntity tempExam = examRepo.findByNameAndRootOrgId(examInfo.getName(),
					examInfo.getRootOrgId());
			if (null != tempExam) {
				throw new StatusException("E-002005", "考试名称已存在");
			}
			exam = new ExamEntity();
			exam.setEnable(true);
			realStatus = DataExecutionStatus.CREATION;
		}
		// (根据考试名称)新增或创建
		else if (es.equals(DataExecutionStatus.CREATION_OR_UPDATE)) {
			if (null != examInfo.getId()) {
				throw new StatusException("E-002006", "id is needless");
			}
			exam = examRepo.findByNameAndRootOrgId(examInfo.getName(), examInfo.getRootOrgId());
			if (null == exam) {
				exam = new ExamEntity();
				exam.setEnable(true);
				realStatus = DataExecutionStatus.CREATION;
			} else {
				if (!exam.getRootOrgId().equals(examInfo.getRootOrgId())) {
					throw new StatusException("E-002003", "rootOrgId is wrong");
				}
				if (!exam.getExamType().equals(examInfo.getExamType())) {
					throw new StatusException("E-002100", "examType is wrong");
				}
				realStatus = DataExecutionStatus.UPDATE;
			}
		}

		if (realStatus.equals(DataExecutionStatus.CREATION)) {
			if (null == examInfo.getBeginTime()) {
				throw new StatusException("E-002006", "beginTime is null");
			}
			if (null == examInfo.getEndTime()) {
				throw new StatusException("E-002006", "endTime is null");
			}
		}

		exam.setExamType(examInfo.getExamType());
		if (null != examInfo.getBeginTime()) {
			exam.setBeginTime(examInfo.getBeginTime());
		}
		if (null != examInfo.getEndTime()) {
			exam.setEndTime(examInfo.getEndTime());
		}
		if (null != examInfo.getDuration()) {
			exam.setDuration(examInfo.getDuration());
		}
		if (null != examInfo.getEnable()) {
			exam.setEnable(examInfo.getEnable());
		}
		if (null != examInfo.getExamTimes()) {
			exam.setExamTimes(examInfo.getExamTimes());
		}
		if (null != examInfo.getExamLimit()) {
			exam.setExamLimit(examInfo.getExamLimit());
		}
		exam.setName(examInfo.getName());
		exam.setRemark(examInfo.getRemark());
		exam.setRootOrgId(examInfo.getRootOrgId());

		ExamEntity saved = examRepo.save(exam);

		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			ExamPropertyEntity entity = examPropertyRepo.findByexamIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new ExamPropertyEntity();
				entity.setExamId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			examPropertyRepo.save(entity);
		}

		SyncExamReq req = new SyncExamReq();
		req.setId(saved.getId());
		req.setBeginTime(saved.getBeginTime());
		req.setDuration(saved.getDuration());
		req.setEnable(saved.getEnable());
		req.setEndTime(saved.getEndTime());
		req.setExamTimes(saved.getExamTimes());
		req.setExamType(saved.getExamType().name());
		req.setName(saved.getName());
		req.setRemark(saved.getRemark());
		req.setRootOrgId(saved.getRootOrgId());
		req.setRootOrgName(rootOrg.getName());
		req.setSyncType("update");
		dataSyncCloudService.syncExam(req);

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param properties
	 * @return
	 */
	private Map<DynamicEnum, String> checkAndGetExamProperties(Map<String, String> properties) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();

		Map<DynamicEnum, String> map = Maps.newHashMap();
		if (null == properties) {
			return map;
		}

		for (Entry<String, String> entry : properties.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			DynamicEnum de = null;
			try {
				de = manager.getByName(key);
			} catch (Exception e) {
				throw new StatusException("E-001004", "考试属性错误. key=" + key);
			}
			if (!de.isLegal(value)) {
				throw new StatusException("E-001200", "考试属性值非法. key=" + key);
			}
			map.put(de, value);
		}

		String beforeExamRemark = properties.get("BEFORE_EXAM_REMARK");
		if (null != beforeExamRemark && beforeExamRemark.length() > 250000) {
			throw new StatusException("E-001002", "考前说明内容过大");
		}

		String afterExamRemark = properties.get("AFTER_EXAM_REMARK");
		if (null != afterExamRemark && afterExamRemark.length() > 250000) {
			throw new StatusException("E-001002", "考后说明内容过大");
		}

		String cheatingRemark = properties.get("CHEATING_REMARK");
		if (null != cheatingRemark && cheatingRemark.length() > 250000) {
			throw new StatusException("E-001002", "作弊说明内容过大");
		}

		return map;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExamOrg(cn.
	 * com.qmth.examcloud.core.examwork.service.bean.ExamOrgInfo)
	 */
	@Override
	public ExamOrgSettingsEntity saveExamOrgSettings(ExamOrgSettingsInfo examOrgInfo) {
		ExamOrgSettingsEntity examOrgEntity = null;

		if (null == examOrgInfo.getId()) {
			examOrgEntity = examOrgSettingsRepo.findByExamIdAndOrgId(examOrgInfo.getExamId(),
					examOrgInfo.getOrgId());
			if (null == examOrgEntity) {
				examOrgEntity = new ExamOrgSettingsEntity();
			}
		} else {
			examOrgEntity = examOrgSettingsRepo.findOne(examOrgInfo.getId());
			if (null == examOrgEntity) {
				throw new StatusException("E-001101", "id is wrong");
			}
		}

		examOrgEntity.setBeginTime(examOrgInfo.getBeginTime());
		examOrgEntity.setEndTime(examOrgInfo.getEndTime());
		examOrgEntity.setRootOrgId(examOrgInfo.getRootOrgId());
		examOrgEntity.setExamId(examOrgInfo.getExamId());
		examOrgEntity.setOrgId(examOrgInfo.getOrgId());
		examOrgEntity.setExamLimit(examOrgInfo.getExamLimit());

		Map<DynamicEnum, String> map = checkAndGetExamProperties(examOrgInfo.getProperties());

		ExamOrgSettingsEntity saved = examOrgSettingsRepo.save(examOrgEntity);

		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			if (StringUtils.isBlank(value)) {
				value = null;
			} else {
				value = value.trim();
			}
			ExamOrgPropertyEntity entity = examOrgPropertyRepo
					.findByexamIdAndOrgIdAndKeyId(saved.getExamId(), saved.getOrgId(), de.getId());
			if (null == entity) {
				entity = new ExamOrgPropertyEntity();
				entity.setExamId(saved.getExamId());
				entity.setOrgId(saved.getOrgId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			examOrgPropertyRepo.save(entity);
		}

		return saved;
	}

	@Override
	public String getOrgProperty(Long examId, Long orgId, String key) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName(key);

		if (null != orgId) {
			ExamOrgPropertyEntity examOrgPropertyEntity = examOrgPropertyRepo
					.findByexamIdAndOrgIdAndKeyId(examId, orgId, de.getId());
			if (null != examOrgPropertyEntity) {
				return examOrgPropertyEntity.getValue();
			}
		}

		ExamPropertyEntity examPropertyEntity = examPropertyRepo.findByexamIdAndKeyId(examId,
				de.getId());
		if (null == examPropertyEntity) {
			return null;
		}

		return examPropertyEntity.getValue();
	}

	@Override
	public List<Map<String, Object>> importExamOrgSettings(Long examId, File file) {

		ExamEntity examEntity = examRepo.findOne(examId);
		List<String[]> lineList = null;
		try {
			lineList = ExcelReader.readSheetBySax(PathUtil.getCanonicalPath(file), 1, 6);
		} catch (Exception e) {
			throw new StatusException("B-200110", "Excel 解析失败");
		}

		if (CollectionUtils.isEmpty(lineList)) {
			throw new StatusException("B-200111", "Excel无内容");
		}

		if (10001 < lineList.size()) {
			throw new StatusException("B-200112", "数据行数不能超过10000");
		}

		List<Map<String, Object>> failRecords = Collections
				.synchronizedList(new ArrayList<Map<String, Object>>());
		List<ExamOrgSettingsEntity> orgSettingsList = Lists.newArrayList();

		for (int i = 0; i < lineList.size(); i++) {
			String[] line = lineList.get(i);
			if (0 == i) {
				if (headerError(line)) {
					throw new StatusException("B-100111", "Excel表头错误");
				}
				continue;
			}

			boolean hasError = false;
			StringBuilder msg = new StringBuilder();

			ExamOrgSettingsEntity orgSettings = new ExamOrgSettingsEntity();
			orgSettings.setRootOrgId(examEntity.getRootOrgId());
			orgSettings.setExamId(examId);

			String orgId = trimAndNullIfBlank(line[0]);
			if (StringUtils.isBlank(orgId)) {
				msg.append("  学习中心ID不能为空");
				hasError = true;
			} else {
				try {
					long orgIdLong = Long.parseLong(orgId);

					GetOrgReq getOrgReq = new GetOrgReq();
					getOrgReq.setOrgId(orgIdLong);
					GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
					OrgBean orgBean = getOrgResp.getOrg();

					if (null == orgBean.getParentId()
							|| !orgBean.getRootId().equals(examEntity.getRootOrgId())) {
						msg.append("  学习中心ID非法");
						hasError = true;
					} else {
						orgSettings.setOrgId(orgIdLong);
					}
				} catch (NumberFormatException e) {
					msg.append("  学习中心ID必须为整数");
					hasError = true;
				}
			}

			String examLimit = trimAndNullIfBlank(line[3]);
			if (StringUtils.isNotBlank(examLimit)) {
				if (examLimit.equals("是")) {
					orgSettings.setExamLimit(false);
				} else if (examLimit.equals("否")) {
					orgSettings.setExamLimit(true);
				} else {
					msg.append("  是否可以开始考试必须为['是','否',空]");
					hasError = true;
				}
			}

			String beginTime = trimAndNullIfBlank(line[4]);
			if (StringUtils.isNotBlank(beginTime)) {
				try {
					Date beginDate = DateUtil.parse(beginTime, DatePatterns.ISO);
					orgSettings.setBeginTime(beginDate);
				} catch (Exception e) {
					msg.append("  开始考试时间格式错误. 正确格式为 " + DatePatterns.ISO);
					hasError = true;
				}
			}
			String endTime = trimAndNullIfBlank(line[5]);
			if (StringUtils.isNotBlank(endTime)) {
				try {
					Date endDate = DateUtil.parse(endTime, DatePatterns.ISO);
					orgSettings.setEndTime(endDate);
				} catch (Exception e) {
					msg.append("  结束考试时间格式错误. 正确格式为 " + DatePatterns.ISO);
					hasError = true;
				}
			}

			if (hasError) {
				failRecords.add(newError(i + 1, msg.toString()));
			} else {
				orgSettingsList.add(orgSettings);
			}

		}

		if (CollectionUtils.isNotEmpty(failRecords)) {
			return failRecords;
		}

		for (ExamOrgSettingsEntity cur : orgSettingsList) {
			ExamOrgSettingsEntity query = examOrgSettingsRepo.findByExamIdAndOrgId(examId,
					cur.getOrgId());

			if (null != query) {
				query.setExamLimit(cur.getExamLimit());
				query.setBeginTime(cur.getBeginTime());
				query.setEndTime(cur.getEndTime());
				examOrgSettingsRepo.save(query);
			} else {
				examOrgSettingsRepo.save(cur);
			}
		}

		return failRecords;

	}

	private Map<String, Object> newError(int lineNum, String msg) {
		Map<String, Object> map = Maps.newHashMap();
		map.put("lineNum", lineNum);
		map.put("msg", msg);
		return map;
	}

	private String trimAndNullIfBlank(String s) {
		if (StringUtils.isBlank(s)) {
			return null;
		}
		return s.trim();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param header
	 */
	private boolean headerError(String[] header) {
		for (int i = 0; i < EXAM_ORG_SETTINGS_EXCEL_HEADER.length; i++) {
			if (null == header[i]) {
				return true;
			}
			if (!EXAM_ORG_SETTINGS_EXCEL_HEADER[i].equals(header[i].trim())) {
				return true;
			}
		}
		return false;
	}

}
